/**
 * @file src/ReadData.cpp
 *
 * @author Youri Hoogstrate
 *
 * @section LICENSE
 * segmentation-fold can predict RNA 2D structures including K-turns.
 * Copyright (C) 2012-2016 Youri Hoogstrate
 *
 * This file is part of segmentation-fold and originally taken from
 * yh-kt-fold.
 *
 * segmentation-fold is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * segmentation-fold is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @todo replace dataset for [21] Mirela Andronescu, Anne Condon, Holger
 * H. Hoos, David H. Mathews and Kevin P. Murphy - Computational
 * approaches for RNA energy parameter estimation; RNA (2010);
 * (doi: 10.1261/rna.1950510)
 *
 * [22] Magnus Alm Rosenblad, Marcela Davilla Lopez, Paul Piccinelli
 * and Tore Samuelsson - Inventory and analysis of the protein subunits
 * of the ribonucleases P and MRP provides further evidence of homology
 * between the yeast and human enzymes; Nucleic Acids Research (2006);
 * (doi: doi:10.1093/nar/gkl626)
 */



#include "Pair.hpp"
#include "Direction.hpp"
#include "Nucleotide.hpp"
#include "Position.hpp"
#include "PairingPlus.hpp"
#include "Direction.hpp"

#include "Sequence.hpp"

#include "Segment.hpp"
#include "SegmentLoop.hpp"
#include "SegmentTreeElement.hpp"
#include "SegmentTree.hpp"
#include "SegmentLoopTree.hpp"
#include "ReadData.hpp"


#include <vector>
#include <algorithm>
#include <map>



/**
 * @brief Initiates a ReadData class.
 */
ReadData::ReadData() :
	loop_hairpin(30 + 1, N_INFINITY),
	loop_bulge(30 + 1, N_INFINITY),
	loop_interior(30 + 1, N_INFINITY)
{
	this->load_int11();
	this->load_int21();
	this->load_int22();
	
	this->load_stack();
	this->load_tstackh();
	this->load_tstacki();
	
	this->load_loop_hairpin();
	this->load_loop_interior();
	this->load_loop_bulge();
	this->load_triloop();
	this->load_tloop();
	
	this->load_miscloop();
	this->load_poppen();
}



/**
 * @brief Load energy values for Asint3
 *
 * @section DESCRIPTION
 * Structure:
 * p2n(j',i') p2n(j,i) n2n(j'-1) n2n(i'-1) n2n(i'-2)
 */
void ReadData::load_int21()
{
	//5': [i]  [ i+1]    [i']
	//     |              |
	//3': [j][j'+1][j'+1][j']
	//3': [j][j -1][j -2][j']
	
	//this->int21[Pairing(i,j)][Pairing(i',j')][Nucleotide i+1][Nucleotide j'+2][Nucleotide j'+1]
	//this->int21[Pairing(i,j)][Pairing(i',j')][Nucleotide i+1][Nucleotide j -1][Nucleotide j -2]
	
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 0.6f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 1.6f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 0.6f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 1.6f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.3f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 0.6f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 1.6f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.3f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 0.6f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.1f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 1.6f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.0f;
	this->int21[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.9f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 1.4f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.4f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 3.3f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 3.3f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.0f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 4.8f;
	this->int21[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 3.9f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 3.6f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.2f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.1f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 3.1f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 3.1f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.8f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 4.0f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 4.0f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 3.2f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.3f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.3f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 3.7f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 3.2f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.8f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 5.5f;
	this->int21[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.7f;
	
}



/*  Deprecated since it isn't used in this program.
void ReadData::loadAsint5()
{

}
*/



/**
 * @brief Load Bulge-loop energies
 *
 * @section DESCRPITION
 * bulge[0], ... ,bulge[29] correspond to bulge loops of size 1,...,30 respectively
 * From 31 and further extrapolation is applied and described in more detail in Zuker::energyBulgeLoop()
 *
 */
void ReadData::load_loop_bulge()
{
	this->loop_bulge[ 0] = N_INFINITY;///@todo redundant because of initiation of ReadData?
	
	this->loop_bulge[ 1] = 3.8f;
	this->loop_bulge[ 2] = 2.8f;
	this->loop_bulge[ 3] = 3.2f;
	this->loop_bulge[ 4] = 3.6f;
	this->loop_bulge[ 5] = 4.0f;
	this->loop_bulge[ 6] = 4.4f;
	this->loop_bulge[ 7] = 4.6f;
	this->loop_bulge[ 8] = 4.7f;
	this->loop_bulge[ 9] = 4.8f;
	this->loop_bulge[10] = 4.9f;
	this->loop_bulge[11] = 5.0f;
	this->loop_bulge[12] = 5.1f;
	this->loop_bulge[13] = 5.2f;
	this->loop_bulge[14] = 5.3f;
	this->loop_bulge[15] = 5.4f;
	this->loop_bulge[16] = 5.4f;
	this->loop_bulge[17] = 5.5f;
	this->loop_bulge[18] = 5.5f;
	this->loop_bulge[19] = 5.6f;
	this->loop_bulge[10] = 5.7f;
	this->loop_bulge[21] = 5.7f;
	this->loop_bulge[22] = 5.8f;
	this->loop_bulge[23] = 5.8f;
	this->loop_bulge[24] = 5.8f;
	this->loop_bulge[25] = 5.9f;
	this->loop_bulge[26] = 5.9f;
	this->loop_bulge[27] = 6.0f;
	this->loop_bulge[28] = 6.0f;
	this->loop_bulge[29] = 6.0f;
	this->loop_bulge[30] = 6.1f;
	
}
/*
void ReadData::loadDangle()
{
	dangle[0][0][0][0] = 0.0f;
	dangle[1][0][0][0] = 0.0f;
	dangle[0][1][0][0] = 0.0f;
	dangle[1][1][0][0] = 0.0f;
	dangle[0][2][0][0] = 0.0f;
	dangle[1][2][0][0] = -1.7f;
	dangle[0][3][0][0] = -0.8f;
	dangle[1][3][0][0] = 0.0f;
	dangle[0][4][0][0] = 0.0f;
	dangle[1][4][0][0] = 0.0f;
	dangle[0][0][1][0] = 0.0f;
	dangle[1][0][1][0] = 0.0f;
	dangle[0][1][1][0] = 0.0f;
	dangle[1][1][1][0] = 0.0f;
	dangle[0][2][1][0] = 0.0f;
	dangle[1][2][1][0] = -0.8f;
	dangle[0][3][1][0] = -0.5f;
	dangle[1][3][1][0] = 0.0f;
	dangle[0][4][1][0] = 0.0f;
	dangle[1][4][1][0] = 0.0f;
	dangle[0][0][2][0] = 0.0f;
	dangle[1][0][2][0] = 0.0f;
	dangle[0][1][2][0] = 0.0f;
	dangle[1][1][2][0] = 0.0f;
	dangle[0][2][2][0] = 0.0f;
	dangle[1][2][2][0] = -1.7f;
	dangle[0][3][2][0] = -0.8f;
	dangle[1][3][2][0] = 0.0f;
	dangle[0][4][2][0] = 0.0f;
	dangle[1][4][2][0] = 0.0f;
	dangle[0][0][3][0] = 0.0f;
	dangle[1][0][3][0] = 0.0f;
	dangle[0][1][3][0] = 0.0f;
	dangle[1][1][3][0] = 0.0f;
	dangle[0][2][3][0] = 0.0f;
	dangle[1][2][3][0] = -1.2f;
	dangle[0][3][3][0] = -0.6f;
	dangle[1][3][3][0] = 0.0f;
	dangle[0][4][3][0] = 0.0f;
	dangle[1][4][3][0] = 0.0f;
	dangle[0][0][4][0] = 0.0f;
	dangle[1][0][4][0] = 0.0f;
	dangle[0][1][4][0] = 0.0f;
	dangle[1][1][4][0] = 0.0f;
	dangle[0][2][4][0] = 0.0f;
	dangle[1][2][4][0] = 0.0f;
	dangle[0][3][4][0] = 0.0f;
	dangle[1][3][4][0] = 0.0f;
	dangle[0][4][4][0] = 0.0f;
	dangle[1][4][4][0] = 0.0f;
	dangle[0][0][0][1] = 0.0f;
	dangle[1][0][0][1] = 0.0f;
	dangle[0][1][0][1] = 0.0f;
	dangle[1][1][0][1] = 0.0f;
	dangle[0][2][0][1] = 0.0f;
	dangle[1][2][0][1] = -0.2f;
	dangle[0][3][0][1] = -0.3f;
	dangle[1][3][0][1] = 0.0f;
	dangle[0][4][0][1] = 0.0f;
	dangle[1][4][0][1] = 0.0f;
	dangle[0][0][1][1] = 0.0f;
	dangle[1][0][1][1] = 0.0f;
	dangle[0][1][1][1] = 0.0f;
	dangle[1][1][1][1] = 0.0f;
	dangle[0][2][1][1] = 0.0f;
	dangle[1][2][1][1] = -0.3f;
	dangle[0][3][1][1] = -0.1f;
	dangle[1][3][1][1] = 0.0f;
	dangle[0][4][1][1] = 0.0f;
	dangle[1][4][1][1] = 0.0f;
	dangle[0][0][2][1] = 0.0f;
	dangle[1][0][2][1] = 0.0f;
	dangle[0][1][2][1] = 0.0f;
	dangle[1][1][2][1] = 0.0f;
	dangle[0][2][2][1] = 0.0f;
	dangle[1][2][2][1] = 0.0f;
	dangle[0][3][2][1] = -0.2f;
	dangle[1][3][2][1] = 0.0f;
	dangle[0][4][2][1] = 0.0f;
	dangle[1][4][2][1] = 0.0f;
	dangle[0][0][3][1] = 0.0f;
	dangle[1][0][3][1] = 0.0f;
	dangle[0][1][3][1] = 0.0f;
	dangle[1][1][3][1] = 0.0f;
	dangle[0][2][3][1] = 0.0f;
	dangle[1][2][3][1] = 0.0f;
	dangle[0][3][3][1] = -0.2f;
	dangle[1][3][3][1] = 0.0f;
	dangle[0][4][3][1] = 0.0f;
	dangle[1][4][3][1] = 0.0f;
	dangle[0][0][4][1] = 0.0f;
	dangle[1][0][4][1] = 0.0f;
	dangle[0][1][4][1] = 0.0f;
	dangle[1][1][4][1] = 0.0f;
	dangle[0][2][4][1] = 0.0f;
	dangle[1][2][4][1] = 0.0f;
	dangle[0][3][4][1] = 0.0f;
	dangle[1][3][4][1] = 0.0f;
	dangle[0][4][4][1] = 0.0f;
	dangle[1][4][4][1] = 0.0f;
	dangle[0][0][0][2] = 0.0f;
	dangle[1][0][0][2] = 0.0f;
	dangle[0][1][0][2] = 0.0f;
	dangle[1][1][0][2] = 0.3f;
	dangle[0][2][0][2] = -2.2f;
	dangle[1][2][0][2] = 0.3f;
	dangle[0][3][0][2] = 0.010f;
	dangle[1][3][0][2] = 0.0f;
	dangle[0][4][0][2] = 5.6f;
	dangle[1][4][0][2] = 5.4f;
	dangle[0][0][1][2] = 6.5f;
	dangle[1][0][1][2] = 6.6f;
	dangle[0][1][1][2] = 6.9f;
	dangle[1][1][1][2] = 7.0f;
	dangle[0][2][1][2] = 7.2f;
	dangle[1][2][1][2] = 7.3f;
	dangle[0][3][1][2] = 7.5f;
	dangle[1][3][1][2] = 7.5f;
	dangle[0][4][1][2] = 7.7f;
	dangle[1][4][1][2] = 0.0f;
	dangle[0][0][2][2] = 1.8f;
	dangle[1][0][2][2] = 2.0f;
	dangle[0][1][2][2] = 2.5f;
	dangle[1][1][2][2] = 2.6f;
	dangle[0][2][2][2] = 3.0f;
	dangle[1][2][2][2] = 3.0f;
	dangle[0][3][2][2] = 3.3f;
	dangle[1][3][2][2] = 3.3f;
	dangle[0][4][2][2] = 3.5f;
	dangle[1][4][2][2] = 3.5f;
	dangle[0][0][3][2] = 3.7f;
	dangle[1][0][3][2] = 0.5f;
	dangle[0][1][3][2] = 1.7f;
	dangle[1][1][3][2] = 1.1f;
	dangle[0][2][3][2] = 1.7f;
	dangle[1][2][3][2] = 1.1f;
	dangle[0][3][3][2] = 1.1f;
	dangle[1][3][3][2] = 1.1f;
	dangle[0][4][3][2] = 1.1f;
	dangle[1][4][3][2] = 1.1f;
	dangle[0][0][4][2] = 1.1f;
	dangle[1][0][4][2] = 1.7f;
	dangle[0][1][4][2] = 1.1f;
	dangle[1][1][4][2] = 1.1f;
	dangle[0][2][4][2] = 1.7f;
	dangle[1][2][4][2] = 1.1f;
	dangle[0][3][4][2] = 1.7f;
	dangle[1][3][4][2] = 1.7f;
	dangle[0][4][4][2] = 1.7f;
	dangle[1][4][4][2] = 1.7f;
	dangle[0][0][0][3] = 1.1f;
	dangle[1][0][0][3] = 1.1f;
	dangle[0][1][0][3] = -0.4f;
	dangle[1][1][0][3] = 1.1f;
	dangle[0][2][0][3] = 1.1f;
	dangle[1][2][0][3] = 1.1f;
	dangle[0][3][0][3] = 1.7f;
	dangle[1][3][0][3] = 1.1f;
	dangle[0][4][0][3] = 1.7f;
	dangle[1][4][0][3] = 1.7f;
	dangle[0][0][1][3] = 1.7f;
	dangle[1][0][1][3] = 1.7f;
	dangle[0][1][1][3] = 1.7f;
	dangle[1][1][1][3] = 1.7f;
	dangle[0][2][1][3] = 0.4f;
	dangle[1][2][1][3] = 1.1f;
	dangle[0][3][1][3] = 0.4f;
	dangle[1][3][1][3] = 0.4f;
	dangle[0][4][1][3] = 1.7f;
	dangle[1][4][1][3] = 1.1f;
	dangle[0][0][2][3] = 1.7f;
	dangle[1][0][2][3] = 1.7f;
	dangle[0][1][2][3] = 1.7f;
	dangle[1][1][2][3] = 1.7f;
	dangle[0][2][2][3] = 1.7f;
	dangle[1][2][2][3] = 1.7f;
	dangle[0][3][2][3] = 1.1f;
	dangle[1][3][2][3] = 1.7f;
	dangle[0][4][2][3] = 0.4f;
	dangle[1][4][2][3] = 0.4f;
	dangle[0][0][3][3] = 1.1f;
	dangle[1][0][3][3] = 0.4f;
	dangle[0][1][3][3] = 1.1f;
	dangle[1][1][3][3] = 1.7f;
	dangle[0][2][3][3] = 1.7f;
	dangle[1][2][3][3] = 1.7f;
	dangle[0][3][3][3] = 1.7f;
	dangle[1][3][3][3] = 1.7f;
	dangle[0][4][3][3] = 1.1f;
	dangle[1][4][3][3] = 1.7f;
	dangle[0][0][4][3] = 1.1f;
	dangle[1][0][4][3] = 1.1f;
	dangle[0][1][4][3] = 1.1f;
	dangle[1][1][4][3] = 0.5f;
	dangle[0][2][4][3] = 1.1f;
	dangle[1][2][4][3] = 1.1f;
	dangle[0][3][4][3] = 1.1f;
	dangle[1][3][4][3] = 1.1f;
	dangle[0][4][4][3] = 1.7f;
	dangle[1][4][4][3] = 1.7f;
	dangle[0][0][0][4] = 1.1f;
	dangle[1][0][0][4] = 1.7f;
	dangle[0][1][0][4] = 1.1f;
	dangle[1][1][0][4] = 1.1f;
	dangle[0][2][0][4] = 1.7f;
	dangle[1][2][0][4] = 1.1f;
	dangle[0][3][0][4] = 1.7f;
	dangle[1][3][0][4] = 1.1f;
	dangle[0][4][0][4] = 1.1f;
	dangle[1][4][0][4] = 1.1f;
	dangle[0][0][1][4] = 1.1f;
	dangle[1][0][1][4] = 1.1f;
	dangle[0][1][1][4] = 1.1f;
	dangle[1][1][1][4] = 1.7f;
	dangle[0][2][1][4] = 1.1f;
	dangle[1][2][1][4] = 1.1f;
	dangle[0][3][1][4] = 1.7f;
	dangle[1][3][1][4] = 1.1f;
	dangle[0][4][1][4] = 1.7f;
	dangle[1][4][1][4] = 1.7f;
	dangle[0][0][2][4] = 1.7f;
	dangle[1][0][2][4] = 1.7f;
	dangle[0][1][2][4] = 1.1f;
	dangle[1][1][2][4] = 1.1f;
	dangle[0][2][2][4] = 0.5f;
	dangle[1][2][2][4] = 1.1f;
	dangle[0][3][2][4] = 1.1f;
	dangle[1][3][2][4] = 1.1f;
	dangle[0][4][2][4] = 1.7f;
	dangle[1][4][2][4] = 1.1f;
	dangle[0][0][3][4] = 1.7f;
	dangle[1][0][3][4] = 1.7f;
	dangle[0][1][3][4] = 1.7f;
	dangle[1][1][3][4] = 1.7f;
	dangle[0][2][3][4] = 1.7f;
	dangle[1][2][3][4] = 1.7f;
	dangle[0][3][3][4] = 0.4f;
	dangle[1][3][3][4] = 1.1f;
	dangle[0][4][3][4] = 0.4f;
	dangle[1][4][3][4] = 0.4f;
	dangle[0][0][4][4] = 1.7f;
	dangle[1][0][4][4] = 1.1f;
	dangle[0][1][4][4] = 1.7f;
	dangle[1][1][4][4] = 1.7f;
	dangle[0][2][4][4] = 1.7f;
	dangle[1][2][4][4] = 1.7f;
	dangle[0][3][4][4] = 1.7f;
	dangle[1][3][4][4] = 1.7f;
	dangle[0][4][4][4] = 1.1f;
	dangle[1][4][4][4] = 1.7f;
}
*/


/*
void ReadData::loadEparam()
{
	this->eparam[0] = 0;												// Seems to be a stack penalty
	this->eparam[1] = 0;												// Seems to be a bulge loop penalty?   dS(T)=dS(30)+ eparam[1] +logParam*ln(n/30)
	//	this->eparam[EPARAM_INTERIOR_LOOP_PENALTY] = 0.0f;					// Seems to be an interior loop penalty
	this->eparam[EPARAM_MISCLOOP_PENALTY]      = 0;						// contribution from miscloop
	this->eparam[4] = 3.4f;												// multibranched loops: offset
	this->eparam[5] = 0;
	this->eparam[6] = 0.3f;
	this->eparam[7] = 0.3f;
	this->eparam[8] = 0.4f;
	this->eparam[EPARAM_AU_PENALTY]            = 0.5;					// terminal AU penalty - is this also GU penalty?!..
	this->eparam[EPARAM_GGG_U_PENALTY]         = -2.2;					// bonus for GGG hairpin

	///@deprecated this->eparam[11] = 0.3f;												// c hairpin slope
	///@deprecated this->eparam[12] = 1.6f;												// c hairpin intercept
	///@deprecated this->eparam[13] = 1.4f;												// c hairpin of 3

	this->eparam[14] = 4.1f;												// Intermolecular initiation free energy
}*/


/**
 * @brief Loads data for the "prelog" parameter.
 *
 * @section DESCRIPTION
 * The prelog parameter is used for extrapolation of loops of size larger than 30 nucleotides.
 *
 *
 * @todo Use data files (XML) for storage.
 */
void ReadData::load_miscloop()
{
	this->miscloop[MISCLOOP_PRELOG] = 1.079f;
	
	this->miscloop[MISCLOOP_ASYMETRIC_INTENRAL_LOOP] = 3.0f;
	
	this->miscloop[MISCLOOP_FM_ARRAY_0] = 0.5f;
	this->miscloop[MISCLOOP_FM_ARRAY_1] = 0.5f;
	this->miscloop[MISCLOOP_FM_ARRAY_2] = 0.5f;
	this->miscloop[MISCLOOP_FM_ARRAY_3] = 0.5f;
	
	this->miscloop[MISCLOOP_MULTIBRANCH_OFFSET] = 3.4f;
	this->miscloop[MISCLOOP_MULTIBRANCH_FB_PENALTY] = 0.0f;
	this->miscloop[MISCLOOP_MULTIBRANCH_HELIX_PENALTY] = 0.4f;
	
	this->miscloop[MISCLOOP_EFN2_OFFSET] = 10.10f;
	this->miscloop[MISCLOOP_EFN2_FB_PENALTY] = -0.30f;
	this->miscloop[MISCLOOP_EFN2_HELIX_PENALTY] = -0.30f;
	
	this->miscloop[MISCLOOP_AU_PENALTY] = 0.5f;							// terminal AU penalty - is this also GU penalty?!..
	this->miscloop[MISCLOOP_GGG_U_PENALTY] = -2.2f;						// bonus for GGG hairpin
	
	this->miscloop[MISCLOOP_C_HAIRPIN_SLOPE] = 0.30f;
	this->miscloop[MISCLOOP_C_HAIRPIN_INTERCEPT] = 1.60f;
	this->miscloop[MISCLOOP_C_HAIRPIN_OF_3] = 1.40f;
	
	this->miscloop[MISCLOOP_INTERMOLECULAR_INIT] = 4.10f;
	this->miscloop[MISCLOOP_GAIL_RULE] = 1;
}



/**
 */
void ReadData::load_loop_hairpin()
{
	this->minimal_hairpin_length = 3;
	
	this->loop_hairpin[ 3] = 5.7f;///@todo rename to loop_hairping
	this->loop_hairpin[ 4] = 5.6f;
	this->loop_hairpin[ 5] = 5.6f;
	this->loop_hairpin[ 6] = 5.4f;
	this->loop_hairpin[ 7] = 5.9f;
	this->loop_hairpin[ 8] = 5.6f;
	this->loop_hairpin[ 9] = 6.4f;
	this->loop_hairpin[10] = 6.5f;
	this->loop_hairpin[11] = 6.6f;
	this->loop_hairpin[12] = 6.7f;
	this->loop_hairpin[13] = 6.8f;
	this->loop_hairpin[14] = 6.9f;
	this->loop_hairpin[15] = 6.9f;
	this->loop_hairpin[16] = 7.0f;
	this->loop_hairpin[17] = 7.1f;
	this->loop_hairpin[18] = 7.1f;
	this->loop_hairpin[19] = 7.2f;
	this->loop_hairpin[20] = 7.2f;
	this->loop_hairpin[21] = 7.3f;
	this->loop_hairpin[22] = 7.3f;
	this->loop_hairpin[23] = 7.4f;
	this->loop_hairpin[24] = 7.4f;
	this->loop_hairpin[25] = 7.5f;
	this->loop_hairpin[26] = 7.5f;
	this->loop_hairpin[27] = 7.5f;
	this->loop_hairpin[28] = 7.6f;
	this->loop_hairpin[29] = 7.6f;
	this->loop_hairpin[30] = 7.7f;
}



/**
 * @brief Loads the interior loops
 **/
void ReadData::load_loop_interior()
{
	this->loop_interior[ 0] = N_INFINITY;///@todo redundant because of initiation of ReadData? if so, do not set
	this->loop_interior[ 1] = N_INFINITY;///@todo redundant because of initiation of ReadData? if so, do not set
	this->loop_interior[ 2] = N_INFINITY;///@todo redundant because of initiation of ReadData? if so, do not set
	this->loop_interior[ 3] = N_INFINITY;///@todo redundant because of initiation of ReadData? if so, do not set
	
	this->loop_interior[ 4] = 1.7f;
	this->loop_interior[ 5] = 1.8f;
	this->loop_interior[ 6] = 2.0f;
	this->loop_interior[ 7] = 2.2f;
	this->loop_interior[ 8] = 2.3f;
	this->loop_interior[ 9] = 2.4f;
	this->loop_interior[10] = 2.5f;
	this->loop_interior[11] = 2.6f;
	this->loop_interior[12] = 2.7f;
	this->loop_interior[13] = 2.8f;
	this->loop_interior[14] = 2.9f;
	this->loop_interior[15] = 3.0f;
	this->loop_interior[16] = 3.0f;
	this->loop_interior[17] = 3.1f;
	this->loop_interior[18] = 3.1f;
	this->loop_interior[19] = 3.2f;
	this->loop_interior[20] = 3.3f;
	this->loop_interior[21] = 3.3f;
	this->loop_interior[22] = 3.4f;
	this->loop_interior[23] = 3.4f;
	this->loop_interior[24] = 3.4f;
	this->loop_interior[25] = 3.5f;
	this->loop_interior[26] = 3.5f;
	this->loop_interior[27] = 3.6f;
	this->loop_interior[28] = 3.6f;
	this->loop_interior[29] = 3.6f;
	this->loop_interior[30] = 3.7f;
}



/**
 * @brief Load the f(m) array ("see Ninio for details")
 *
 * @attention more details: http://www.bioinfo.rpi.edu/zukerm/rna/energy/efiles-3.0-rna/#MISC
 */
void ReadData::load_poppen()
{
	this->poppen_p.clear();
	this->poppen_p.push_back(0.0);// spacer element - if this is accessed, a bulge loop has gone inside the interior loop function
	this->poppen_p.push_back(0.5);// elem 1
	this->poppen_p.push_back(0.5);// elem 2
	this->poppen_p.push_back(0.5);// elem 3
	this->poppen_p.push_back(0.5);// elem 4
}



/**
 * @brief Load structure for int11 (previously known as sint2)
 *
 * @section DESCRIPTION
 * Structure of sint2:
 *
 * p2n(i,j) p2n(i',j') n2n(i+1) n2n(j-1)
 *
 */
void ReadData::load_int11()
{
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C] = -0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C] = 0.5f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.7f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U] = -0.3f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int11[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U] = 0.4f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G] = -2.1f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int11[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U] = -0.7f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A] = -0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C] = 0.5f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.7f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C] = 0.5f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int11[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U] = -0.3f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int11[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int11[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U] = 1.7f;
}


/**
 * @brief Structure of data should be: int22[pairing(i,j)][pairing(i',j')][i+1][j-1]][i+2]][j-2]];
 *
 */
void ReadData::load_int22()
{
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.6f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -3.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = -0.2f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::AU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::AU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.8f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = -0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = -0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -3.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = -0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -1.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.6f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -4.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -4.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::CG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -4.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.5f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.4f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.9f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::CG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.6f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -1.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -1.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -2.6f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -1.6f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -4.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -4.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = -0.8f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = -0.7f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -1.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -1.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -4.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -4.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.2f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.3f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = -1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -1.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = -1.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = -0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -4.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.8f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -4.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.7f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -3.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.2f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.6f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -1.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = -0.2f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.2f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -1.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = -0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = -0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = -0.2f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -1.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.2f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = -0.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = -0.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GC][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.6f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -3.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = -0.2f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::GU][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.4f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.8f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.8f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.3f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::GU][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.6f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -3.5f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UA][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.7f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UA][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::AU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = -0.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = -1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.2f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.6f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -2.2f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = -0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = -0.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = -0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -3.5f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.1f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UG][PairingType::CG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = -0.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -1.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 0.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 0.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.7f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 0.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -3.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.1f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.4f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.2f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::GC][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 0.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.8f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.7f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 0.9f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::GU][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UA][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 3.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 3.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 0.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 1.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::A][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 1.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::G][Nucleotide::U] = -0.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 1.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 3.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::C][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 1.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 1.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 2.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 1.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::G][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 2.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::G] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 1.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.1f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::G][Nucleotide::U] = -1.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::G] = -0.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::G][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 1.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::C][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::G][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::A][Nucleotide::U][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::A] = 3.4f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::A] = 2.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::C][Nucleotide::U] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::G] = 2.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::C] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::C][Nucleotide::U][Nucleotide::U] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::A] = 1.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::C] = -0.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::G] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::A] = -0.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::C][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::A] = 0.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::G] = 0.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::G][Nucleotide::U] = -2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::C] = 0.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::G] = -1.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::G][Nucleotide::U][Nucleotide::U] = 0.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::A] = 2.9f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::C] = 1.7f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::G] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::A][Nucleotide::U] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::A] = 1.8f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::G] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::C][Nucleotide::U] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::A] = 2.5f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::C] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::G] = 2.2f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::G][Nucleotide::U] = 1.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::A] = 2.0f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::C] = 1.6f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::G] = 2.3f;
	this->int22[PairingType::UG][PairingType::UG][Nucleotide::U][Nucleotide::U][Nucleotide::U][Nucleotide::U] = 0.8f;
}


/**
 */
void ReadData::load_stack()
{
	this->stack[PairingType::UA][PairingType::UA] = -0.90f;
	this->stack[PairingType::GC][PairingType::UA] = -2.20f;
	this->stack[PairingType::CG][PairingType::UA] = -2.10f;
	this->stack[PairingType::UG][PairingType::UA] = -0.60f;
	this->stack[PairingType::AU][PairingType::UA] = -1.10f;
	this->stack[PairingType::GU][PairingType::UA] = -1.40f;
	this->stack[PairingType::UA][PairingType::GC] = -2.10f;
	this->stack[PairingType::GC][PairingType::GC] = -3.30f;
	this->stack[PairingType::CG][PairingType::GC] = -2.40f;
	this->stack[PairingType::UG][PairingType::GC] = -1.40f;
	this->stack[PairingType::AU][PairingType::GC] = -2.10f;
	this->stack[PairingType::GU][PairingType::GC] = -2.10f;
	this->stack[PairingType::UA][PairingType::CG] = -2.40f;
	this->stack[PairingType::GC][PairingType::CG] = -3.40f;
	this->stack[PairingType::CG][PairingType::CG] = -3.30f;
	this->stack[PairingType::UG][PairingType::CG] = -1.50f;
	this->stack[PairingType::AU][PairingType::CG] = -2.20f;
	this->stack[PairingType::GU][PairingType::CG] = -2.50f;
	this->stack[PairingType::UA][PairingType::UG] = -1.30f;
	this->stack[PairingType::GC][PairingType::UG] = -2.50f;
	this->stack[PairingType::CG][PairingType::UG] = -2.10f;
	this->stack[PairingType::UG][PairingType::UG] = -0.50f;
	this->stack[PairingType::AU][PairingType::UG] = -1.40f;
	this->stack[PairingType::GU][PairingType::UG] = 1.30f;
	this->stack[PairingType::UA][PairingType::AU] = -1.30f;
	this->stack[PairingType::GC][PairingType::AU] = -2.40f;
	this->stack[PairingType::CG][PairingType::AU] = -2.10f;
	this->stack[PairingType::UG][PairingType::AU] = -1.0f;
	this->stack[PairingType::AU][PairingType::AU] = -0.90f;
	this->stack[PairingType::GU][PairingType::AU] = -1.30f;
	this->stack[PairingType::UA][PairingType::GU] = -1.0f;
	this->stack[PairingType::GC][PairingType::GU] = -1.50f;
	this->stack[PairingType::CG][PairingType::GU] = -1.40f;
	this->stack[PairingType::UG][PairingType::GU] = 0.30f;
	this->stack[PairingType::AU][PairingType::GU] = -0.60f;
	this->stack[PairingType::GU][PairingType::GU] = -0.50f;
}


/**
 */
void ReadData::load_tloop()
{
	this->tloop_map[Sequence("GGGGAC")] = 3.00f;
	this->tloop_map[Sequence("CGAAAG")] = 3.00f;
	this->tloop_map[Sequence("GGAGAC")] = 3.00f;
	this->tloop_map[Sequence("CGCAAG")] = 3.00f;
	this->tloop_map[Sequence("GGAAAC")] = 3.00f;
	this->tloop_map[Sequence("CGGAAG")] = 3.00f;
	this->tloop_map[Sequence("CUUCGG")] = 3.00f;
	this->tloop_map[Sequence("CGUGAG")] = 3.00f;
	this->tloop_map[Sequence("CGAAGG")] = 2.50f;
	this->tloop_map[Sequence("CUACGG")] = 2.50f;
	this->tloop_map[Sequence("GGCAAC")] = 2.50f;
	this->tloop_map[Sequence("CGCGAG")] = 2.50f;
	this->tloop_map[Sequence("UGAGAG")] = 2.50f;
	this->tloop_map[Sequence("CGAGAG")] = 2.00f;
	this->tloop_map[Sequence("AGAAAU")] = 2.00f;
	this->tloop_map[Sequence("CGUAAG")] = 2.00f;
	this->tloop_map[Sequence("CUAACG")] = 2.00f;
	this->tloop_map[Sequence("UGAAAG")] = 2.00f;
	this->tloop_map[Sequence("GGAAGC")] = 1.50f;
	this->tloop_map[Sequence("GGGAAC")] = 1.50f;
	this->tloop_map[Sequence("UGAAAA")] = 1.50f;
	this->tloop_map[Sequence("AGCAAU")] = 1.50f;
	this->tloop_map[Sequence("AGUAAU")] = 1.50f;
	this->tloop_map[Sequence("CGGGAG")] = 1.50f;
	this->tloop_map[Sequence("AGUGAU")] = 1.50f;
	this->tloop_map[Sequence("GGCGAC")] = 1.50f;
	this->tloop_map[Sequence("GGGAGC")] = 1.50f;
	this->tloop_map[Sequence("GUGAAC")] = 1.50f;
	this->tloop_map[Sequence("UGGAAA")] = 1.50f;
}


/**
 * @brief Loads triloops
 *
 * @note no triloops available
 *
 */
void ReadData::load_triloop()
{
}


/**
 * @brief Loads tstackh (terminal stack for hairpin)
 *
 * @section DESCRIPTION
 *  Access: tstkh[PairingType(i,j)][Nucleotide i + 1][Nucleotide j - 1]
 *
 */
void ReadData::load_tstackh()
{
	this->tstackh[PairingType::UA][Nucleotide::A][Nucleotide::A] = -0.50f;
	this->tstackh[PairingType::GC][Nucleotide::A][Nucleotide::A] = -1.10f;
	this->tstackh[PairingType::CG][Nucleotide::A][Nucleotide::A] = -1.50f;
	this->tstackh[PairingType::UG][Nucleotide::A][Nucleotide::A] = -0.50f;
	this->tstackh[PairingType::AU][Nucleotide::A][Nucleotide::A] = -0.30f;
	this->tstackh[PairingType::GU][Nucleotide::A][Nucleotide::A] = 0.20f;
	this->tstackh[PairingType::UA][Nucleotide::C][Nucleotide::A] = -0.20f;
	this->tstackh[PairingType::GC][Nucleotide::C][Nucleotide::A] = -1.10f;
	this->tstackh[PairingType::CG][Nucleotide::C][Nucleotide::A] = -1.0f;
	this->tstackh[PairingType::UG][Nucleotide::C][Nucleotide::A] = -0.20f;
	this->tstackh[PairingType::AU][Nucleotide::C][Nucleotide::A] = -0.10f;
	this->tstackh[PairingType::GU][Nucleotide::C][Nucleotide::A] = -0.10f;
	this->tstackh[PairingType::UA][Nucleotide::G][Nucleotide::A] = -1.40f;
	this->tstackh[PairingType::GC][Nucleotide::G][Nucleotide::A] = -2.40f;
	this->tstackh[PairingType::CG][Nucleotide::G][Nucleotide::A] = -2.20f;
	this->tstackh[PairingType::UG][Nucleotide::G][Nucleotide::A] = -0.80f;
	this->tstackh[PairingType::AU][Nucleotide::G][Nucleotide::A] = -1.10f;
	this->tstackh[PairingType::GU][Nucleotide::G][Nucleotide::A] = -0.90f;
	this->tstackh[PairingType::UA][Nucleotide::U][Nucleotide::A] = -0.30f;
	this->tstackh[PairingType::GC][Nucleotide::U][Nucleotide::A] = -1.90f;
	this->tstackh[PairingType::CG][Nucleotide::U][Nucleotide::A] = -1.70f;
	this->tstackh[PairingType::UG][Nucleotide::U][Nucleotide::A] = -0.60f;
	this->tstackh[PairingType::AU][Nucleotide::U][Nucleotide::A] = -0.30f;
	this->tstackh[PairingType::GU][Nucleotide::U][Nucleotide::A] = -0.30f;
	this->tstackh[PairingType::UA][Nucleotide::A][Nucleotide::C] = -0.30f;
	this->tstackh[PairingType::GC][Nucleotide::A][Nucleotide::C] = -1.50f;
	this->tstackh[PairingType::CG][Nucleotide::A][Nucleotide::C] = -1.50f;
	this->tstackh[PairingType::UG][Nucleotide::A][Nucleotide::C] = -0.30f;
	this->tstackh[PairingType::AU][Nucleotide::A][Nucleotide::C] = -0.50f;
	this->tstackh[PairingType::GU][Nucleotide::A][Nucleotide::C] = -0.50f;
	this->tstackh[PairingType::UA][Nucleotide::C][Nucleotide::C] = -0.10f;
	this->tstackh[PairingType::GC][Nucleotide::C][Nucleotide::C] = -0.70f;
	this->tstackh[PairingType::CG][Nucleotide::C][Nucleotide::C] = -0.90f;
	this->tstackh[PairingType::UG][Nucleotide::C][Nucleotide::C] = -0.10f;
	this->tstackh[PairingType::AU][Nucleotide::C][Nucleotide::C] = -0.20f;
	this->tstackh[PairingType::GU][Nucleotide::C][Nucleotide::C] = -0.20f;
	this->tstackh[PairingType::UA][Nucleotide::G][Nucleotide::C] = -1.20f;
	this->tstackh[PairingType::GC][Nucleotide::G][Nucleotide::C] = -2.90f;
	this->tstackh[PairingType::CG][Nucleotide::G][Nucleotide::C] = -2.0f;
	this->tstackh[PairingType::UG][Nucleotide::G][Nucleotide::C] = -1.20f;
	this->tstackh[PairingType::AU][Nucleotide::G][Nucleotide::C] = -1.20f;
	this->tstackh[PairingType::GU][Nucleotide::G][Nucleotide::C] = -1.10f;
	this->tstackh[PairingType::UA][Nucleotide::U][Nucleotide::C] = -0.10f;
	this->tstackh[PairingType::GC][Nucleotide::U][Nucleotide::C] = -1.0f;
	this->tstackh[PairingType::CG][Nucleotide::U][Nucleotide::C] = -1.40f;
	this->tstackh[PairingType::UG][Nucleotide::U][Nucleotide::C] = -0.10f;
	this->tstackh[PairingType::AU][Nucleotide::U][Nucleotide::C] = -0.30f;
	this->tstackh[PairingType::GU][Nucleotide::U][Nucleotide::C] = -0.30f;
	this->tstackh[PairingType::UA][Nucleotide::A][Nucleotide::G] = -0.60f;
	this->tstackh[PairingType::GC][Nucleotide::A][Nucleotide::G] = -1.30f;
	this->tstackh[PairingType::CG][Nucleotide::A][Nucleotide::G] = -1.40f;
	this->tstackh[PairingType::UG][Nucleotide::A][Nucleotide::G] = -0.60f;
	this->tstackh[PairingType::AU][Nucleotide::A][Nucleotide::G] = -0.30f;
	this->tstackh[PairingType::GU][Nucleotide::A][Nucleotide::G] = -0.30f;
	this->tstackh[PairingType::UA][Nucleotide::C][Nucleotide::G] = -1.20f;
	this->tstackh[PairingType::GC][Nucleotide::C][Nucleotide::G] = -2.40f;
	this->tstackh[PairingType::CG][Nucleotide::C][Nucleotide::G] = -2.90f;
	this->tstackh[PairingType::UG][Nucleotide::C][Nucleotide::G] = -1.70f;
	this->tstackh[PairingType::AU][Nucleotide::C][Nucleotide::G] = -1.50f;
	this->tstackh[PairingType::GU][Nucleotide::C][Nucleotide::G] = -1.50f;
	this->tstackh[PairingType::UA][Nucleotide::G][Nucleotide::G] = -0.70f;
	this->tstackh[PairingType::GC][Nucleotide::G][Nucleotide::G] = -1.40f;
	this->tstackh[PairingType::CG][Nucleotide::G][Nucleotide::G] = -1.60f;
	this->tstackh[PairingType::UG][Nucleotide::G][Nucleotide::G] = -0.30f;
	this->tstackh[PairingType::AU][Nucleotide::G][Nucleotide::G] = -0.20f;
	this->tstackh[PairingType::GU][Nucleotide::G][Nucleotide::G] = -0.30f;
	this->tstackh[PairingType::UA][Nucleotide::U][Nucleotide::G] = -0.50f;
	this->tstackh[PairingType::GC][Nucleotide::U][Nucleotide::G] = -2.20f;
	this->tstackh[PairingType::CG][Nucleotide::U][Nucleotide::G] = -1.80f;
	this->tstackh[PairingType::UG][Nucleotide::U][Nucleotide::G] = -0.60f;
	this->tstackh[PairingType::AU][Nucleotide::U][Nucleotide::G] = -0.60f;
	this->tstackh[PairingType::GU][Nucleotide::U][Nucleotide::G] = -0.40f;
	this->tstackh[PairingType::UA][Nucleotide::A][Nucleotide::U] = -0.50f;
	this->tstackh[PairingType::GC][Nucleotide::A][Nucleotide::U] = -2.10f;
	this->tstackh[PairingType::CG][Nucleotide::A][Nucleotide::U] = -1.80f;
	this->tstackh[PairingType::UG][Nucleotide::A][Nucleotide::U] = -0.50f;
	this->tstackh[PairingType::AU][Nucleotide::A][Nucleotide::U] = -0.30f;
	this->tstackh[PairingType::GU][Nucleotide::A][Nucleotide::U] = -0.30f;
	this->tstackh[PairingType::UA][Nucleotide::C][Nucleotide::U] = 0;
	this->tstackh[PairingType::GC][Nucleotide::C][Nucleotide::U] = -0.50f;
	this->tstackh[PairingType::CG][Nucleotide::C][Nucleotide::U] = -0.80f;
	this->tstackh[PairingType::UG][Nucleotide::C][Nucleotide::U] = 0;
	this->tstackh[PairingType::AU][Nucleotide::C][Nucleotide::U] = -0.20f;
	this->tstackh[PairingType::GU][Nucleotide::C][Nucleotide::U] = -0.20f;
	this->tstackh[PairingType::UA][Nucleotide::G][Nucleotide::U] = -0.20f;
	this->tstackh[PairingType::GC][Nucleotide::G][Nucleotide::U] = -1.20f;
	this->tstackh[PairingType::CG][Nucleotide::G][Nucleotide::U] = -1.10f;
	this->tstackh[PairingType::UG][Nucleotide::G][Nucleotide::U] = -0.70f;
	this->tstackh[PairingType::AU][Nucleotide::G][Nucleotide::U] = 0.20f;
	this->tstackh[PairingType::GU][Nucleotide::G][Nucleotide::U] = 0;
	this->tstackh[PairingType::UA][Nucleotide::U][Nucleotide::U] = -0.80f;
	this->tstackh[PairingType::GC][Nucleotide::U][Nucleotide::U] = -1.50f;
	this->tstackh[PairingType::CG][Nucleotide::U][Nucleotide::U] = -2.0f;
	this->tstackh[PairingType::UG][Nucleotide::U][Nucleotide::U] = -0.80f;
	this->tstackh[PairingType::AU][Nucleotide::U][Nucleotide::U] = -1.10f;
	this->tstackh[PairingType::GU][Nucleotide::U][Nucleotide::U] = -1.10f;
}


/**
 * @brief Terminal stack interior loop
 *
 */
void ReadData::load_tstacki()
{
	this->tstki[PairingType::AU][Nucleotide::A][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::A][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::A][Nucleotide::G] = -0.40f;
	this->tstki[PairingType::AU][Nucleotide::A][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::C][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::C][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::C][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::C][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::G][Nucleotide::A] = -0.40f;
	this->tstki[PairingType::AU][Nucleotide::G][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::G][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::G][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::U][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::U][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::U][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::AU][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::A][Nucleotide::G] = -1.10f;
	this->tstki[PairingType::CG][Nucleotide::A][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::C][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::G][Nucleotide::A] = -1.10f;
	this->tstki[PairingType::CG][Nucleotide::G][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::U][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::U][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::CG][Nucleotide::U][Nucleotide::U] = -0.70f;
	this->tstki[PairingType::GC][Nucleotide::A][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::A][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::A][Nucleotide::G] = -1.10f;
	this->tstki[PairingType::GC][Nucleotide::A][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::C][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::C][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::C][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::C][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::G][Nucleotide::A] = -1.10f;
	this->tstki[PairingType::GC][Nucleotide::G][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::G][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::G][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::U][Nucleotide::A] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::U][Nucleotide::C] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::U][Nucleotide::G] = 0.0f;
	this->tstki[PairingType::GC][Nucleotide::U][Nucleotide::U] = -0.70f;
	this->tstki[PairingType::GU][Nucleotide::A][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::A][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::A][Nucleotide::G] = -0.40f;
	this->tstki[PairingType::GU][Nucleotide::A][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::C][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::C][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::C][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::C][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::G][Nucleotide::A] = -0.40f;
	this->tstki[PairingType::GU][Nucleotide::G][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::G][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::G][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::U][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::U][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::U][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::GU][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::UA][Nucleotide::A][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::A][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::A][Nucleotide::G] = -0.40f;
	this->tstki[PairingType::UA][Nucleotide::A][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::C][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::C][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::C][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::C][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::G][Nucleotide::A] = -0.40f;
	this->tstki[PairingType::UA][Nucleotide::G][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::G][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::G][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::U][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::U][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::U][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UA][Nucleotide::U][Nucleotide::U] = 0.0f;
	this->tstki[PairingType::UG][Nucleotide::A][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::A][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::A][Nucleotide::G] = -0.40f;
	this->tstki[PairingType::UG][Nucleotide::A][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::C][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::C][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::C][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::C][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::G][Nucleotide::A] = -0.40f;
	this->tstki[PairingType::UG][Nucleotide::G][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::G][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::G][Nucleotide::U] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::U][Nucleotide::A] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::U][Nucleotide::C] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::U][Nucleotide::G] = 0.70f;
	this->tstki[PairingType::UG][Nucleotide::U][Nucleotide::U] = 0.0f;
}
